import jwt

from ievv_auth.ievv_api_key.models import ScopedAPIKey
from ievv_auth.ievv_jwt.backends.base_backend import AbstractBackend
from ievv_auth.ievv_jwt.exceptions import JWTBackendError


class ApiKeyBackend(AbstractBackend):

    @classmethod
    def get_backend_name(cls):
        return 'api-key'

    def __init__(self, api_key_id, *args, **kwargs):
        super(ApiKeyBackend, self).__init__(*args, **kwargs)
        try:
            self.api_key_instance = ScopedAPIKey.objects.get(id=api_key_id)
        except ScopedAPIKey.DoesNotExist:
            raise JWTBackendError('API key is not valid')

    def make_payload(self):
        payload = super(ApiKeyBackend, self).make_payload()
        api_key_scope_payload = self.api_key_instance.base_jwt_payload
        api_key_scope_payload['api_key_id'] = self.api_key_instance.id
        api_key_scope_payload.update(**payload)
        return api_key_scope_payload

    @classmethod
    def make_instance_from_raw_jwt(cls, raw_jwt):
        payload = jwt.decode(jwt=raw_jwt, verify=False)
        return cls(api_key_id=payload['api_key_id'])

    def make_authenticate_success_response(self, *args, **kwargs):
        return {
            'access': self.encode()
        }



from rest_framework import serializers
from rest_framework.exceptions import AuthenticationFailed

from ievv_auth.ievv_api_key.models import ScopedAPIKey
from ievv_auth.ievv_jwt.backends.backend_registry import JWTBackendRegistry
from ievv_auth.ievv_jwt.exceptions import JWTBackendError


class ApiKeyObtainJWTSerializer(serializers.Serializer):
    api_key = serializers.CharField()

    def validate(self, attrs):
        is_valid, instance = ScopedAPIKey.objects.is_valid_with_logging(api_key=attrs['api_key'])
        if not is_valid:
            raise AuthenticationFailed('Api key has expired or is invalid')
        jwt_backend_class = JWTBackendRegistry.get_instance().get_backend(instance.jwt_backend_name)
        if not jwt_backend_class:
            raise AuthenticationFailed('Unknown jwt backend could not authenticate')
        try:
            backend = jwt_backend_class(api_key_id=instance.id)
        except JWTBackendError:
            raise AuthenticationFailed('Api key is invalid')
        return backend.make_authenticate_success_response()
